#include "femNeumann.h"
#include "femProblem.h"
#include "femGroup.h"

namespace fem
{

Neumann::Neumann(Problem &pbl, std::string const &groupname,
                 std::string const &dof, double val)
    : m_nodetag(0), m_dofname(dof), m_val(val)
{
    try
    {
        group = pbl.groups_by_name.at(groupname);
    }
    catch (const std::exception &e)
    {
        throw std::runtime_error("Neumann: group \"" + groupname + "\" not found!");
    }
    pbl.neumanns.push_back(this);
}

Neumann::Neumann(Problem &pbl, size_t nodetag,
                 std::string const &dof, double val)
    : group(nullptr), m_nodetag(nodetag), m_dofname(dof), m_val(val)
{
    pbl.neumanns.push_back(this);
}

Neumann::~Neumann()
{
    // if(debug_level()>0)
    //     std::cout << "~Neumann()\n";
}

FEM_API std::ostream &
operator<<(std::ostream &out, Neumann const &obj)
{
    if (obj.group)
        out << "Neumann BC on \"" << obj.group->name << "\": "
            << obj.m_dofname << '=' << obj.m_val;
    // else
    //     out << "Neumann BC on node " << obj.m_nodetag << ": "
    //         << obj.m_dofname << '=' << obj.m_val;

    return out;
}

} // namespace fem
